from collections import Counter
import pprint
import json
import os
import numpy as np
from jarvis.core.atoms import Atoms
from collections import OrderedDict
from jarvis.core.kpoints import generate_kgrid, Kpoints3D


class Poscar(object):
    def __init__(self, atoms, comment="System"):
        self.atoms = atoms
        self.comment = comment

    @staticmethod
    def from_file(filename="POSCAR"):
        with open(filename, "r") as f:
            return Poscar.from_string(f.read())

    def write_file(self, filename):
        f = open(filename, "w")
        header = (
            str(self.comment)
            + str("\n1.0\n")
            + str(self.atoms.lattice_mat[0][0])
            + " "
            + str(self.atoms.lattice_mat[0][1])
            + " "
            + str(self.atoms.lattice_mat[0][2])
            + "\n"
            + str(self.atoms.lattice_mat[1][0])
            + " "
            + str(self.atoms.lattice_mat[1][1])
            + " "
            + str(self.atoms.lattice_mat[1][2])
            + "\n"
            + str(self.atoms.lattice_mat[2][0])
            + " "
            + str(self.atoms.lattice_mat[2][1])
            + " "
            + str(self.atoms.lattice_mat[2][2])
            + "\n"
        )
        order = np.argsort(self.atoms.elements)
        coords = self.atoms.frac_coords
        coords_ordered = np.array(coords)[order]
        elements_ordered = np.array(self.atoms.elements)[order]
        props_ordered = np.array(self.atoms.props)[order]
        check_selective_dynamics = False
        if "T" in "".join(map(str, self.atoms.props[0])):
            middle = (
                " ".join(map(str, Counter(elements_ordered).keys()))
                + "\n"
                + " ".join(map(str, Counter(elements_ordered).values()))
                + "\nSelective dynamics\n"
                + "Direct\n"
            )
        else:
            middle = (
                " ".join(map(str, Counter(elements_ordered).keys()))
                + "\n"
                + " ".join(map(str, Counter(elements_ordered).values()))
                + "\ndirect\n"
            )
        rest = ""
        # print ('repr',self.frac_coords, self.frac_coords.shape)
        for ii, i in enumerate(coords_ordered):
            rest = rest + " ".join(map(str, i)) + " " + str(props_ordered[ii]) + "\n"

        result = header + middle + rest

        f.write(result)
        f.close()

    @staticmethod
    def from_string(lines):
        text = lines.splitlines()
        comment = text[0]
        scale = float(text[1])
        lattice_mat = []
        lattice_mat.append([float(i) for i in text[2].split()])
        lattice_mat.append([float(i) for i in text[3].split()])
        lattice_mat.append([float(i) for i in text[4].split()])
        lattice_mat = np.array(lattice_mat)

        uniq_elements = text[5].split()
        element_count = np.array([int(i) for i in text[6].split()])
        elements = []
        for i, ii in enumerate(element_count):
            for j in range(ii):
                elements.append(uniq_elements[i])
        cartesian = True
        if "d" in text[7] or "D" in text[7]:
            cartesian = False
        # print ('cartesian poscar=',cartesian,text[7])
        num_atoms = int(np.sum(element_count))
        coords = []
        for i in range(num_atoms):
            coords.append([float(i) for i in text[8 + i].split()[0:3]])
        coords = np.array(coords)
        atoms = Atoms(
            lattice_mat=lattice_mat,
            coords=coords,
            elements=elements,
            cartesian=cartesian,
        )
        # print (atoms)
        formula = atoms.composition.formula

        return Poscar(atoms, comment=comment)

    def __repr__(self):
        header = (
            str(self.comment)
            + str("\n1.0\n")
            + str(self.atoms.lattice_mat[0][0])
            + " "
            + str(self.atoms.lattice_mat[0][1])
            + " "
            + str(self.atoms.lattice_mat[0][2])
            + "\n"
            + str(self.atoms.lattice_mat[1][0])
            + " "
            + str(self.atoms.lattice_mat[1][1])
            + " "
            + str(self.atoms.lattice_mat[1][2])
            + "\n"
            + str(self.atoms.lattice_mat[2][0])
            + " "
            + str(self.atoms.lattice_mat[2][1])
            + " "
            + str(self.atoms.lattice_mat[2][2])
            + "\n"
        )
        order = np.argsort(self.atoms.elements)
        coords = self.atoms.frac_coords
        coords_ordered = np.array(coords)[order]
        elements_ordered = np.array(self.atoms.elements)[order]
        props_ordered = np.array(self.atoms.props)[order]
        check_selective_dynamics = False
        if "T" in "".join(map(str, self.atoms.props[0])):
            middle = (
                " ".join(map(str, Counter(elements_ordered).keys()))
                + "\n"
                + " ".join(map(str, Counter(elements_ordered).values()))
                + "\nSelective dynamics\n"
                + "Direct\n"
            )
        else:
            middle = (
                " ".join(map(str, Counter(elements_ordered).keys()))
                + "\n"
                + " ".join(map(str, Counter(elements_ordered).values()))
                + "\ndirect\n"
            )
        rest = ""
        # print ('repr',self.frac_coords, self.frac_coords.shape)
        for ii, i in enumerate(coords_ordered):
            rest = rest + " ".join(map(str, i)) + " " + str(props_ordered[ii]) + "\n"
        result = header + middle + rest
        return result


class Incar(object):
    def __init__(self, tags={}):
        self._tags = tags

    @staticmethod
    def from_file(filename="INCAR"):
        with open(filename, "r") as f:
            return Incar.from_string(f.read())

    def update(self, d={}):
        print("selftags1=", self._tags)
        if self._tags != {}:
            for i, j in self._tags.items():
                self._tags[i] = j  # .strip(' ')
        for i, j in d.items():
            self._tags[i] = j
        print("selftags2=", self._tags)
        return Incar(self._tags)

    def get(self, key="POTIM", temp=0.5):
        if key in list(self._tags.keys()):
            return self._tags[key]
        else:
            self._tags[key] = temp
            print("Setting the temp value to the key", temp, key)
            return self._tags[key]

    def to_dict(self):
        return self._tags

    def from_dict(self, data={}):
        print("data", data)
        return Incar(tags=data)

    @staticmethod
    def from_string(lines):
        text = lines.splitlines()
        tags = OrderedDict()
        for i in text:
            if "=" in i:
                tmp = i.split("=")
                tags.setdefault(tmp[0].strip(" "), tmp[1].strip(" "))

        return Incar(tags=tags)

    def __repr__(self):
        return str(self._tags)

    def write_file(self, filename):
        tags = self._tags
        lines = ""
        for i, j in tags.items():
            lines = lines + str(i) + str("=") + str(j) + "\n"
        f = open(filename, "w")
        f.write(lines)
        f.close()


class IndividualPotcarData(object):
    def __init__(self, data={}):
        self._data = data

    def from_string(lines):
        text = lines.splitlines()
        individual_data = OrderedDict()
        individual_data["header1"] = text[0]
        individual_data["header2"] = text[2]
        individual_data["VRHFIN"] = text[3].split("=")[1]
        individual_data["element"] = text[3].split("=")[1].split(":")[0]
        individual_data["LEXCH"] = text[4].split("=")[1].replace(" ", "")
        individual_data["EATOM"] = text[5].split("=")[1].split()[0].replace(" ", "")
        individual_data["TITEL"] = text[7].split("=")[1].replace(" ", "")

        return IndividualPotcarData(data=individual_data)

    def from_file(filename="POTCAR"):
        with open(filename, "r") as f:
            return IndividualPotcarData.from_string(f.read())

    def __repr__(self):
        return pprint.pformat(self._data, indent=4)


class Potcar(object):
    def __init__(
        self,
        elements=[],
        pot_type="POT_GGA_PAW_PBE",
        pot_json_path="",
        potcar_strings={},
    ):
        self._elements = elements
        self._pot_type = pot_type
        self._potcar_strings = potcar_strings
        self._pot_json_path = pot_json_path

        if self._potcar_strings == {}:
            pot_json_file = str(
                os.path.join(os.path.dirname(__file__), "default_potcars.json")
            )
            pot_json = open(pot_json_file, "r")
            pot_json_selected = json.load(pot_json)
            pot_json.close()
            potcar_strings = OrderedDict()
            for i in self._elements:
                for j, k in pot_json_selected.items():
                    if i == j:
                        potcar_strings.setdefault(i, k)
            self._potcar_strings = potcar_strings
            if len(self._elements) != len(self._potcar_strings.keys()):
                raise ValueError(
                    "Number of elements is not same as potcar_strings",
                    self._elements,
                    self._potcar_strings.keys(),
                )
        else:
            pot_json = open(self._pot_json_path, "r")
            pot_json_selected = json.load(pot_json)
            pot_json.close()
            potcar_strings = OrderedDict()
            for i, j in pot_json_selected.items():
                if i in self._elements:
                    potcar_strings.setdefault(i, j)
            self._potcar_strings = potcar_strings
            if len(self._elements) != len(self._potcar_strings.keys()):
                raise ValueError("Number of elements is not same as potcar_strings")

    def catenate_potcar_files(self, destination_filename="POTCAR", filenames=[]):
        with open(destination_filename, "w") as outfile:
            for fname in filenames:
                with open(fname) as infile:
                    for line in infile:
                        outfile.write(line)

    def list_potcar_files(self):
        pot_files = []
        vasp_dir = str(os.environ["JARVIS_VASP_PSP_DIR"])
        vasp_psp_dir = str(os.path.join(vasp_dir, self._pot_type))
        potcar_strings = self._potcar_strings
        for i, j in potcar_strings.items():
            tmp = os.path.join(vasp_psp_dir, j, "POTCAR")
            pot_files.append(tmp)
        return pot_files

    def write_file(self, filename="POTCAR"):
        pot_files = self.list_potcar_files()
        self.catenate_potcar_files(destination_filename=filename, filenames=pot_files)

    def __repr__(self):
        return str(str(self._pot_type) + "\n" + str(self._potcar_strings))


class Kpoints(object):
    def __init__(self, filename=""):
        self.filename = filename
        if filename != "":
            f = open(self.filename, "r")
            self.lines = f.read().splitlines()
            f.close()
            self.kpoints = self.read(self.lines)

    @classmethod
    def read(self, lines):

        if lines[1] == "0":
            return self.get_mesh_kp(lines=lines)
        elif "Reciprocal" in lines[2]:
            return self.get_ibz_kp(lines=lines)
        else:
            ValueError("K-point scheme is not implemented")

    def get_mesh_kp(lines=""):
        print("lines", lines)
        grid = [int(i) for i in lines[3].split()]
        # print (grid)
        kpts = generate_kgrid(grid)
        kpoints = Kpoints3D(kpoints=np.array(kpts))
        return kpoints

    def get_ibz_kp(lines=""):
        kp_labels = []
        all_kp = []
        kp_labels_points = []
        for ii, i in enumerate(lines):
            if ii > 2:
                tmp = i.split()
                k = [tmp[0], tmp[1], tmp[2]]
                all_kp.append(k)
                if len(tmp) == 5:
                    tmp = str("$") + str(tmp[4]) + str("$")
                    if len(kp_labels) == 0:
                        kp_labels.append(tmp)
                        kp_labels_points.append(ii - 3)
                    elif tmp != kp_labels[-1]:
                        kp_labels.append(tmp)
                        kp_labels_points.append(ii - 3)
        labels = []
        for i in range(len(all_kp)):
            labels.append("")
        for i, j in zip(kp_labels, kp_labels_points):
            labels[j] = i
        all_kp = np.array(all_kp, dtype="float")
        kpoints = Kpoints3D(kpoints=all_kp, labels=labels, kpoint_mode="linemode")
        return kpoints
        # print ('kp_labels',labels,len(all_kp))
        # print ('kp2',kp_labels_points, kp_labels,np.array(all_kp,dtype='float'))

        # return kp_labels_points, kp_labels,np.array(all_kp,dtype='float')


# """
if __name__ == "__main__":

    kp = open(
        "../../examples/vasp/SiOptb88/MAIN-RELAX-bulk@mp_149/KPOINTS", "r"
    )  # .read_file()
    lines = kp.read().splitlines()
    kp.close()
    print('lbl',Kpoints.read(lines).labels)
    # print (kp.kpoints)
    import sys

    sys.exit()
    kp = Kpoints(filename="../../examples/vasp/SiOptb88/MAIN-RELAX-bulk@mp_149/KPOINTS")

    p = Poscar.from_file(
        filename="/rk2/knc6/JARVIS-DFT/2D-1L/POSCAR-mp-2815-1L.vasp_PBEBO/MAIN-RELAX-Surf-mp-2815/POSCAR"
    )
    p.write_file("POSCAR")
    from jarvis.db.jsonutils import dumpjson

    dumpjson(p.atoms.to_dict(), "pp.json")
    # print (p.atoms*[3,3,3])
    # p = Incar.from_file(filename='/rk2/knc6/JARVIS-DFT/2D-1L/POSCAR-mp-2815-1L.vasp_PBEBO/MAIN-RELAX-Surf-mp-2815/INCAR')
    # print (p)
    # p.write_file('pp')
    # elements=['S','P']
    # p=Potcar(elements=elements)
    # print (p)
    # p.write_file('POTCAR')
    # inp=IndividualPotcarData.from_file('POTCAR')
    # print (inp)
# """



from jarvis.analysis.defects.vacancy import Vacancy
from jarvis.core.atoms import Atoms


def test_vacancy():
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    vacs = Vacancy(atoms=Si).generate_defects(enforce_c_size=5.0)
    # print ( len(vacs),(vacs[0].to_dict()["defect_structure"].num_atoms))
    assert (len(vacs), vacs[0].to_dict()["defect_structure"].num_atoms) == (1, 127)


# test_vacancy()



from jarvis.analysis.interface.zur import ZSLGenerator, mismatch_strts, get_hetero
from jarvis.core.atoms import Atoms
from jarvis.io.vasp.inputs import Poscar
import os

s1 = Poscar.from_file(os.path.join(os.path.dirname(__file__), "POSCAR-JVASP-652"))
s2 = Poscar.from_file(os.path.join(os.path.dirname(__file__), "POSCAR-JVASP-664"))
s3 = Poscar.from_file(os.path.join(os.path.dirname(__file__), "POSCAR-JVASP-688"))


def test_zur():
    info = mismatch_strts(film=s2.atoms, subs=s3.atoms)
    a = info['film_sl'].center_around_origin()
    b = info['subs_sl'].center_around_origin()
    combined  = get_hetero(a,b,seperation=3).center_around_origin()
    print (combined)
    info = mismatch_strts(film=s1.atoms, subs=s2.atoms)
    a = info['film_sl'].center_around_origin()
    b = info['subs_sl'].center_around_origin()
    combined  = get_hetero(a,b,seperation=3).center_around_origin()
    print (combined)
    assert (round(info["mismatch_u"], 3), round(info["mismatch_angle"], 3)) == (
        0.002,
        0.0,
    )


test_zur()



from jarvis.core.atoms import Atoms
import os
import json
import shutil


class Wannier90win(object):
    def __init__(
        self,
        struct="",
        dis_num_iter=100,
        kmesh_tol=0.000000001,
        dis_mix_ratio=0.5,
        num_iter=100,
        num_print_cycles=50,
        frozen_tol=2,
        efermi=None,
        soc=True,
        semi_core_states=None,
    ):
        self.struct = struct
        self.efermi = efermi
        self.soc = soc
        self.dis_num_iter = dis_num_iter
        self.dis_mix_ratio = dis_mix_ratio
        self.num_iter = num_iter
        self.num_print_cycles = num_print_cycles
        self.frozen_tol = frozen_tol
        self.semi_core_states = semi_core_states
        self.kmesh_tol = kmesh_tol
        if self.semi_core_states is None:
            path_semi_core = str(
                os.path.join(os.path.dirname(__file__), "default_semicore.json")
            )
            f = open(path_semi_core, "r")
            semi_core_states = json.load(f)
            f.close()
            self.semi_core_states = semi_core_states

    def write_win(self, name="win.input"):

        if self.soc == True:
            mult = 2
        else:
            mult = 1
        semi_core = self.semi_core_states

        xx = self.struct.composition.to_dict()
        elements = []
        ELEMENTCOUNT = []
        for i, j in xx.items():
            elements.append(i)
            ELEMENTCOUNT.append(int(j))
        nwan = 0
        nelectrons = 0
        exclude = 0
        projections = ""
        c = 0
        for e in elements:
            s = semi_core[e]
            exclude += s[0] * ELEMENTCOUNT[c]
            nwan += mult * s[3] * ELEMENTCOUNT[c]
            nelectrons += s[1] * ELEMENTCOUNT[c]
            projections += e + ":" + s[2] + "\n"
            c += 1
        print("excluded", exclude)
        nwan_tot = nwan + exclude
        print("nwan_tot", nwan_tot)
        print("projections", projections)
        if exclude == 0:
            exclude_st = ""
        else:
            exclude_st = "exclude_bands     = 1-" + str(int(exclude)) + "\n"

        f = open(name, "w")
        line = "guiding_centres = TRUE \n"
        f.write(line)
        line = "kmesh_tol=" + str(self.kmesh_tol) + "\n"
        f.write(line)
        # line=str('num_wann =')+str(nwan_tot)+'\n'
        line = str("num_wann =") + str(nwan) + "\n"
        f.write(line)
        f.write(exclude_st)

        tol = self.frozen_tol
        if self.efermi is not None:
            dmax = self.efermi - tol
            dmin = self.efermi + tol
            line = str("dis_froz_max =") + str(dmax) + str("\n")
            f.write(line)
            line = str("dis_froz_min =") + str(dmin) + str("\n")
            f.write(line)

        line = str("dis_num_iter =") + str(self.dis_num_iter) + str("\n")
        f.write(line)
        line = str("dis_mix_ratio =") + str(self.dis_mix_ratio) + str("\n")
        f.write(line)
        line = str("num_iter =") + str(self.num_iter) + str("\n")
        f.write(line)
        line = str("num_print_cycles =") + str(self.num_print_cycles) + str("\n")
        f.write(line)
        line = str("begin projections") + str("\n")
        f.write(line)
        f.write(projections)
        line = str("end projections") + str("\n")
        f.write(line)
        f.close()
        return nwan, exclude

    def write_hr_win(
        self,
        hr_tag="hr_plot",
        prev_win="wannier90.win",
        hr="hr_wannier.win",
        nbands=18,
        soc="",
    ):
        f = open(prev_win, "r")
        lines = f.read().splitlines()
        f.close()
        bak = str(prev_win) + str(".bak")
        shutil.copy2(prev_win, bak)

        # line=str('write_hr=.true. \n')
        line = str(hr_tag) + str(".true. \n")

        f = open(hr, "w")
        f.write(line)
        line = str("num_bands = ") + str(nbands) + ("\n")
        f.write(line)
        for i in lines:
            line = str(i) + str("\n")
            f.write(line)
        f.close()

"""
if __name__ == "__main__":
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    Wannier90win(struct=Si, efermi=0.0).write_win()
    Wannier90win().write_hr_win(prev_win="win.input")
"""



import joblib
from monty.serialization import loadfn, MontyDecoder
from jarvis.core.atoms import Atoms


def monty_to_joblib(
    read_file="/cluster/users/knc6/JARVIS-DFT1/mem-dir1L-8-22-2019.json",
    target_file="test.json",
):
    d = loadfn(read_file, cls=MontyDecoder)
    mem = []
    for i in d:
        info = {}
        jid = i["jid"]
        strt = i["data"][0]["contcar"]
        lattice = strt.lattice.matrix
        fracs = strt.frac_coords
        elements = [j.symbol for j in strt.species]
        atms = Atoms(lattice_mat=lattice, coords=fracs, elements=elements)
        phi = i["phi"]

        info["phi"] = phi
        info["atoms"] = atms
        info["jid"] = jid
        mem.append(info)
    f = open("MonolayersJOBLIB", "wb")
    joblib.dump(mem, f, protocol=2)
    f.close()


monty_to_joblib()



from jarvis.core.kpoints import Kpoints3D, generate_kpath, generate_kgrid
from jarvis.core.atoms import Atoms


def test_kp():
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    lattice_mat = Si.lattice_mat
    kp = Kpoints3D().automatic_length_mesh(lattice_mat=lattice_mat, length=20)
    sym = kp.high_symm_path(Si)._path
    x, y = kp.interpolated_points(Si)
    kpath = generate_kpath(kpath = [[0,0,0],[0,0.5,.5]],num_k=5)
    kps = generate_kgrid(grid=[5, 5, 5])
    
    # for i,j in zip(x,y):
    #   print (i,j)
    # print (len(x))
    assert (len(x), x[0][0], y[0], kpath[0][0],kps[0][0]) == (166, 0, "\Gamma", 0.0, 0.0)

# test_kp()



from monty.serialization import loadfn
d=loadfn('/home/kamalch/anaconda3/envs/cgcnn/lib/python3.6/site-packages/pymatgen/core/periodic_table.json')
dd=loadfn('Elementsold.json')
allkeys=dd['Pr'].keys()
gg=dd

no_there=[]
for i in d.keys():
 if i not in dd.keys():
    no_there.append(i)


for i in no_there:
   gg.setdefault(i,{})

for i in no_there:
  for j in allkeys:
    gg[i][j]=-9999
    for k in d[i].keys():
       if d[i][k]=='no data':
         d[i][k]=-9999
    gg[i]['Z']=d[i]['Atomic no']
    gg[i]['atom_mass']=d[i]['Atomic mass']
    gg[i]['atom_rad']=d[i]['Atomic radius calculated']
    gg[i]['X']=d[i]['X']


import json
f=open('Elements_new.json','w')
f.write(json.dumps(gg))
f.close()

print (len(gg['Eu'].keys()))
print (len(gg['P'].keys()))


